#' Parse the group.by argument and prepare a grouped view of the input table.
#' @param .g list of temporary objects needed.
#' @return Updated version of .g.
.jk.parse.group.by <- function(.g){
	# requires:
	#	group.by	
	#	mysql.connection
	#	input.table
	# assigns:
	#	group.by.ids
	#	group.by.ids.sql
	#	grouped.table
	#	table.with.group.ids
	# creates:
	#	_level_combinations
	#	_groups
	
	# group.by is a list of entries of the form
	#	list(dimension, ..., dimension),
	# where each "dimension" is a list of the form
	#   list(name="group-by variable name", label="...", groups=list(group, ..., group)),
	# each "group" is a list of the form
	#   list(level=level.no, label="...", condition="where.condition.in.x")
	# and level.no is a one-based no. of the hierarchical level, where 1 is the outmost level.

    if (length(.g$group.by)==0) .g$group.by <- list(list(name="all", label="all", groups=list(list(level=1, label="all", condition="TRUE"))))
    
	# extract group-by variable names:
	
	.g$group.by.ids <- 
		g <- unlist(lapply(.g$group.by,function(gbi){ cpaste("id__",gbi$name) }))
		
	.g$group.by.ids.sql <- 
		cpaste(.g$group.by.ids)


	# prepare db:
	
	dbSendQuery(.g$mysql.connection, "DROP TABLE IF EXISTS _level_combinations" )
	dbSendQuery(.g$mysql.connection, "DROP TABLE IF EXISTS _groups" )


	# now construct hierarchical group ids, applicable conditions and sql code parts:

	# no. of level combinations = product of max. level in each group-by variable:
	n.levelcombs <- prod(unlist(lapply( .g$group.by, function(gbi){
			max(unlist(lapply( gbi$groups, function(gri){gri$level} )))
		} )))
	# no. of combined groups = product of no. of groups in each group-by variable:
	n.groups <- prod(unlist(lapply( .g$group.by, function(gbi){length(gbi$groups)} )))
	yet.levelcombs <- 1
	yet.groups <- 1
	levelcombs.list <- list()
	groups.list <- list()
	code.parts <- list()
	
	for (i in 1:length(.g$group.by)) { # for each group-by variable gbi:
	
		gbi <- .g$group.by[[i]]
		name <- gbi$name
		groups <- gbi$groups
		this.groups <- length(groups)
		maxlevel <- max(unlist(lapply( groups, function(gbi){gbi$level} )))
		# init group id vector with a 0 for each level:
		id <- rep(0,maxlevel)
		.g$group.by[[i]]$code.parts <- list()
		.g$group.by[[i]]$ids <- list()
		
		for (j in 1:this.groups) { # for each group grj in gbi:
		
			grj <- groups[[j]]
			# replace placeholder "x" in condition by name of group-by variable:
			.g$group.by[[i]]$groups[[j]]$where <- condition <- replace.word("x",name,grj$condition) 
			level <- grj$level
			# determine hierarchical id vector of group by increasing entry at current level
			# and resetting entries at later levels:
			id[[level]] <- id[[level]]+1 
			if (level<maxlevel) id[(level+1):maxlevel] <- rep(0,maxlevel-level)
			# build id string as d*2-digit hex representation of id vector:
			.g$group.by[[i]]$ids[[j]] <- thisid <- npaste(sprintf("%02X",id[1:level]))
			if (j<length(groups) && level<groups[[j+1]]$level) { # group has sub-group(s):
				.g$group.by[[i]]$code.parts[[j]] <- cpaste(
				
					" WHEN ",condition," THEN CASE "
				)
			} else { # group has no sub-groups:
				code.part <- cpaste(
				
					" WHEN ",condition," THEN '",thisid,"'"
				)
				nextlevel <- ifelse(j<length(groups),groups[[j+1]]$level,1)
				if (level>nextlevel) 
				 for (k in (level-1):nextlevel) 
				  if (id[[k]]>0) { # include implicit "other" sub-group:
					code.part <- cpaste(code.part,
							
					" ELSE '",npaste(sprintf("%02X",id[1:k])),"' END "
					
					) 
				  }
				.g$group.by[[i]]$code.parts[[j]] <- code.part
			}
		}
		code.parts[[i]] <- npaste(.g$group.by[[i]]$code.parts)
		
		# construct a df of level combinations:
		# each row is a level combination, and each group-by variable gets a col
		# which contains the respective no. of initial bytes of the id string for that group-by variable:
		
		levelcombs.list[[cpaste("len_id__",name)]] <- 
			rep(
				 rep(
				 	 2*(1:maxlevel)
				 	,rep(yet.levelcombs,maxlevel))
				,n.levelcombs/yet.levelcombs/maxlevel
			)
		yet.levelcombs <- yet.levelcombs * maxlevel
		
		# construct a df of combined groups.
		# each row is a combined group, and each group-by variable gets a col
		# which contains the respective group id for that group-by variable:
		
		groups.list[[ g[[i]] ]] <- 
			rep(
				 rep(
				 	 unlist(.g$group.by[[i]]$ids)
				 	,rep(n.groups/yet.groups/this.groups,this.groups))
				,yet.groups
			)
		yet.groups <- yet.groups * this.groups
	} 
	levelcombs.df <- data.frame(levelcombs.list)
	groups.df <- data.frame(groups.list)
	
	if (.g$.jk.debug) {
		print("code.parts")
		print(code.parts)
		print("levelcombs")
		print(levelcombs.df)	
		print("groups")
		print(groups.df)
	}	
	
	dbWriteTable(.g$mysql.connection,"_level_combinations",levelcombs.df,overwrite=TRUE)
	dbWriteTable(.g$mysql.connection,"_groups",groups.df,overwrite=TRUE)
	

	# a view of the table with additional cols including full group ids:
	
	.g$table.with.group.ids <- 
		cpaste(	
		 " ( SELECT *, "
			,cpaste(
				 " CASE "
					,code.parts
				," ELSE NULL END AS full_",g
			)
		," FROM ",.g$input.table," AS t ) "
	)
	
	
	# construct a view which distributes table's rows into each fitting group.
	# contains a row for each combination of original table row and combined group that row fits in,
	# so that a by-group-processing gives also all marginals:
	
	# Future work: optimize performance with indices and/or hints in what follows?
	.g$grouped.table <- 
		cpaste(	
		 " ( SELECT t.*, "
			,cpaste(
				" SUBSTR(full_",g,",1,len_",g,") AS ",g
			)
		," FROM _level_combinations AS l, ",.g$table.with.group.ids," AS t "
		," WHERE "
			,paste(
				 " LENGTH(full_",g,")>=len_",g
				,sep="",collapse=" AND "
			)
		," ) "
	) 

	if (.g$.jk.debug) {
		print("table.with.group.ids")
		print(.g$table.with.group.ids)
		#print(dbGetQuery(.g$mysql.connection,.g$table.with.group.ids))
		print("grouped.table")
		print(.g$grouped.table)
		print(dbGetQuery(.g$mysql.connection,.g$grouped.table))
		print(dbGetQuery(.g$mysql.connection,cpaste("select count(*) from ",.g$grouped.table," as s")))
	}
    return(.g)
}
